@@ -3,9 +3,15 @@
  int main(int argc, char *argv[])
 {
 	STRUCT_STAT st;
+	int ret;
 
 	while (--argc > 0) {
-		if (stat(*++argv, &st) < 0) {
+#ifdef USE_STAT64_FUNCS
+		ret = stat64(*++argv, &st);
+#else
+		ret = stat(*++argv, &st);
+#endif
+		if (ret < 0) {
 			fprintf(stderr, "Unable to stat `%s'\n", *argv);
 			exit(1);
 		}
